Package: Dune
Title: Improving replicability in single-cell RNA-Seq cell type
        discovery
Version: 1.23.0
Authors@R: c(person("Hector", "Roux de Bezieux", role = c("aut", "cre"),
                   email = "hector.rouxdebezieux@berkeley.edu",
		   comment = c(ORCID = "0000-0002-1489-8339")),
	    person("Kelly","Street", role = "aut", email = "street.kelly@gmail.com"))
Description: Given a set of clustering labels, Dune merges pairs of clusters to increase mean ARI between labels, improving replicability. 
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.1.1
Imports: BiocParallel, SummarizedExperiment, utils, ggplot2, dplyr,
        tidyr, RColorBrewer, magrittr, gganimate, purrr, aricode
Suggests: knitr, rmarkdown, testthat (>= 2.1.0)
VignetteBuilder: knitr
Depends: R (>= 3.6)
biocViews: Clustering, GeneExpression, RNASeq, Software, SingleCell,
        Transcriptomics, Visualization
git_url: https://git.bioconductor.org/packages/Dune
git_branch: devel
git_last_commit: 9318153
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 21:36:39 UTC; biocbuild
Author: Hector Roux de Bezieux [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1489-8339>),
  Kelly Street [aut]
Maintainer: Hector Roux de Bezieux <hector.rouxdebezieux@berkeley.edu>
