# Generated by roxygen2: do not edit by hand

export(ARIImp)
export(ARIs)
export(ARItrend)
export(ConfusionEvolution)
export(ConfusionPlot)
export(Dune)
export(NMIImp)
export(NMIs)
export(NMItrend)
export(clusterConversion)
export(functionTracking)
export(intermediateMat)
export(plotARIs)
export(plotNMIs)
export(plotPrePost)
export(whenToStop)
exportMethods(Dune)
import(RColorBrewer)
import(dplyr)
import(ggplot2)
import(tidyr)
import(utils)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpparam)
importFrom(RColorBrewer,brewer.pal)
importFrom(SummarizedExperiment,colData)
importFrom(aricode,ARI)
importFrom(aricode,NMI)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,n_distinct)
importFrom(gganimate,transition_states)
importFrom(magrittr,"%>%")
importFrom(purrr,map_df)
importFrom(tidyr,gather)
