---
name: Bug report
about: File an issue to help us improve
title: ''
labels: ''
assignees: ''

---

<!-- Thank you for filing an issue to help us improve Dune! -->
## Expected Behavior


## Actual Behavior


## Reproduction

<!-- If possible, please open a PR with a test that reproduces the problem.
      Adding tests is easy, see  https://github.com/ocaml/dune/blob/master/doc/hacking.rst -->

- PR with a reproducing test: 

<!-- Otherwise, please detail how the behavior can be reproduced -->

1.
1.
1.

## Specifications

- Version of `dune` (output of `dune --version`):
- Version of `ocaml` (output of `ocamlc --version`)
- Operating system (distribution and version):

<!-- remove the following if not used -->
## Additional information 

- Link to gist with verbose output (run `dune` with the `--verbose` flag):
