---
author: Marek Kubica
---

OCaml Package Management With Dune
==================================

:::{warning}
Dune Package Management is not final yet and details are still subject to
change.
:::

In this tutorial we will be looking at how to use Dune for managing project
dependencies. This enables users to install the compiler as well as third-party
dependencies using a single tool which takes care of building code and
dependencies.

To get started you only need Dune. Head to {doc}`setup` to begin the setup.

:::{toctree}
:hidden:
:maxdepth: 1
setup
dependencies
pinning
repos
:::
