Conclusion
==========

This tutorial has covered the parts of Dune that users are the most likely to
interact with:

- the {doc}`/reference/dune/executable`, {doc}`/reference/dune/library`, and
  {doc}`/reference/dune/test` stanzas;
- how to use {doc}`cram tests</reference/cram>` and the workflow of
  {doc}`promotion </concepts/promotion>`;
- bindings to C code using {doc}`foreign stubs </reference/foreign-stubs>`;
- using a ppx deriver.

## Where to go from here

You might be interested in:

- our {doc}`how-to guides </howto/index>` to apply this to your projects;
- various {doc}`explanations </explanation/index>` about how Dune works.
