% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{ARIImp}
\alias{ARIImp}
\title{ARI improvement}
\usage{
ARIImp(merger, unclustered = NULL)
}
\arguments{
\item{merger}{the result from having run \code{\link{Dune}}
on the dataset}

\item{unclustered}{The value assigned to unclustered cells. Default to \code{NULL}}
}
\value{
a vector with the mean ARI between methods at each merge
}
\description{
Compute the ARI improvement over the ARI merging procedure
}
\examples{
data("clusMat", package = "Dune")
merger <- Dune(clusMat = clusMat)
plot(0:nrow(merger$merges), ARIImp(merger))
}
\seealso{
ARItrend
}
