% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.adjustedRandIndex}
\alias{.adjustedRandIndex}
\title{adjustedRandIndex}
\usage{
.adjustedRandIndex(tab)
}
\arguments{
\item{tab}{The confusion matrix}
}
\value{
The ARI
}
\description{
adjustedRandIndex
}
